@@ -20,10 +20,11 @@ from random import choice, sample, shuffle
 from re import sub
 from subprocess import CalledProcessError
 from sys import exit as sysexit
-from sys import stdin, stdout, stderr
+from sys import stderr, stdin, stdout
 from xml.sax.xmlreader import InputSource
 
 from jwt.exceptions import PyJWTError
+from milc import set_metadata  # this function needed to set metadata immediately below
 from pygments import highlight
 from pygments.formatters import Terminal256Formatter
 from pygments.lexers import get_lexer_by_name, load_lexer_from_file
@@ -35,12 +36,11 @@ from yaml import parser
 from netfoundry import __version__ as netfoundry_version
 
 from .exceptions import NeedUserInput, NFAPINoCredentials
-from .network import Networks, Network
+from .network import Network, Networks
 from .network_group import NetworkGroup
 from .organization import Organization
-from .utility import DC_PROVIDERS, MUTABLE_NET_RESOURCES, MUTABLE_RESOURCE_ABBREV, RESOURCE_ABBREV, RESOURCES, is_jwt, normalize_caseless, plural, singular
+from .utility import DC_PROVIDERS, EMBED_NET_RESOURCES, MUTABLE_NET_RESOURCES, MUTABLE_RESOURCE_ABBREV, RESOURCE_ABBREV, RESOURCES, is_jwt, normalize_caseless, plural, singular
 
-from milc import set_metadata  # this function needed to set metadata immediately below
 set_metadata(version=f"v{netfoundry_version}", author="NetFoundry", name="nfctl")  # must precend import milc.cli
 from milc import cli, questions  # this uses metadata set above
 from milc.subcommand import config  # this creates the config subcommand
@@ -163,21 +163,39 @@ def login(cli):
                 table = tabulate(tabular_data=summary_table, headers=['Domain', 'Summary'], tablefmt=table_borders)
                 if cli.config.general.color:
                     highlighted = highlight(table, text_lexer, Terminal256Formatter(style=cli.config.general.style))
-                    cli.echo(highlighted)
+                    try:
+                        cli.echo(highlighted)
+                    except ValueError:
+                        print(highlighted)
                 else:
-                    cli.echo(table)
+                    try:
+                        cli.echo(table)
+                    except ValueError:
+                        print(table)
             elif cli.args.output == "yaml":
                 if cli.config.general.color:
                     highlighted = highlight(yaml_dumps(summary_object, indent=4), yaml_lexer, Terminal256Formatter(style=cli.config.general.style))
-                    cli.echo(highlighted)
+                    try:
+                        cli.echo(highlighted)
+                    except ValueError:
+                        print(highlighted)
                 else:
-                    cli.echo(yaml_dumps(summary_object, indent=4))
+                    try:
+                        cli.echo(yaml_dumps(summary_object, indent=4))
+                    except ValueError:
+                        print(yaml_dumps(summary_object, indent=4))
             elif cli.args.output == "json":
                 if cli.config.general.color:
                     highlighted = highlight(json_dumps(summary_object, indent=4), json_lexer, Terminal256Formatter(style=cli.config.general.style))
-                    cli.echo(highlighted)
+                    try:
+                        cli.echo(highlighted)
+                    except ValueError:
+                        print(highlighted)
                 else:
-                    cli.echo(json_dumps(summary_object, indent=4))
+                    try:
+                        cli.echo(json_dumps(summary_object, indent=4))
+                    except ValueError:
+                        print(json_dumps(summary_object, indent=4))
         else:             # if eval
             nonf = """
 # helper function logs out from NetFoundry
@@ -212,9 +230,15 @@ export MOPURL="{organization.audience}"
 """
             if cli.config.general.color:
                 highlighted = highlight(token_env, bash_lexer, Terminal256Formatter(style=cli.config.general.style))
-                cli.echo(highlighted)
+                try:
+                    cli.echo(highlighted)
+                except ValueError:
+                    print(highlighted)
             else:
-                cli.echo(token_env)
+                try:
+                    cli.echo(token_env)
+                except ValueError:
+                    print(token_env)
 
 
 @cli.subcommand('logout your identity for the current current profile')
@@ -238,7 +262,8 @@ def logout(cli):
 
 
 @cli.argument('query', arg_only=True, action=StoreDictKeyPair, nargs='?', help="id=UUIDv4 or query params as k=v,k=v comma-separated pairs")
-@cli.argument('resource_type', arg_only=True, help='type of resource', metavar="RESOURCE_TYPE", choices=[choice for group in [[singular(type), MUTABLE_NET_RESOURCES[type].abbreviation] for type in MUTABLE_NET_RESOURCES.keys()] for choice in group])
+@cli.argument('resource_type', arg_only=True, help='type of resource', metavar="RESOURCE_TYPE",
+              choices=[choice for group in [[singular(type), MUTABLE_NET_RESOURCES[type].abbreviation] for type in MUTABLE_NET_RESOURCES.keys()] for choice in group])
 # this allows us to pass the edit subcommand's cli object to function get without further modifying that functions params
 @cli.subcommand('duplicate a resource')
 def copy(cli):
@@ -253,6 +278,7 @@ def copy(cli):
         cli.args.resource_type = singular(MUTABLE_RESOURCE_ABBREV[cli.args.resource_type].name)
     spinner.text = f"Getting {cli.args.resource_type} for copying"
     cli.args['accept'] = 'create'
+    cli.args['embed'] = None
     cli.args['output'] = 'text'       # implies tty which allows INFO messages
     with spinner:
         edit_resource_object, network, network_group, organization = get(cli, echo=False, spinner=spinner)
@@ -268,7 +294,8 @@ def copy(cli):
 
 
 @cli.argument('-f', '--file', help='JSON or YAML file', type=argparse.FileType('r', encoding='UTF-8'))
-@cli.argument('resource_type', arg_only=True, help='type of resource', metavar="RESOURCE_TYPE", choices=[choice for group in [[singular(type), RESOURCES[type].abbreviation] for type in MUTABLE_NET_RESOURCES.keys()] for choice in group])
+@cli.argument('resource_type', arg_only=True, help='type of resource', metavar="RESOURCE_TYPE",
+              choices=[choice for group in [[singular(type), RESOURCES[type].abbreviation] for type in MUTABLE_NET_RESOURCES.keys()] for choice in group])
 @cli.subcommand('create a resource from a file')
 def create(cli):
     """Create a resource.
@@ -340,7 +367,8 @@ def create(cli):
 
 
 @cli.argument('query', arg_only=True, action=StoreDictKeyPair, nargs='?', help="id=UUIDv4 or query params as k=v,k=v comma-separated pairs")
-@cli.argument('resource_type', arg_only=True, help='type of resource', metavar="RESOURCE_TYPE", choices=[choice for group in [[singular(type), RESOURCES[type].abbreviation] for type in MUTABLE_NET_RESOURCES.keys()] for choice in group])
+@cli.argument('resource_type', arg_only=True, help='type of resource', metavar="RESOURCE_TYPE",
+              choices=[choice for group in [[singular(type), RESOURCES[type].abbreviation] for type in MUTABLE_NET_RESOURCES.keys()] for choice in group])
 # this allows us to pass the edit subcommand's cli object to function get without further modifying that functions params
 @cli.subcommand('edit a resource with EDITOR')
 def edit(cli):
@@ -354,6 +382,7 @@ def edit(cli):
     if MUTABLE_RESOURCE_ABBREV.get(cli.args.resource_type):
         cli.args.resource_type = singular(MUTABLE_RESOURCE_ABBREV[cli.args.resource_type].name)
     cli.args['accept'] = None
+    cli.args['embed'] = None
     spinner.text = f"Getting {cli.args.resource_type} for editing"
     cli.log.debug(f"opening {cli.args.resource_type} for editing")
     with spinner:
@@ -373,21 +402,24 @@ def edit(cli):
 @cli.argument('query', arg_only=True, action=StoreDictKeyPair, nargs='?', help="id=UUIDv4 or query params as k=v,k=v comma-separated pairs")
 @cli.argument('-k', '--keys', arg_only=True, action=StoreListKeys, help="list of keys as a,b,c to print only selected keys (columns)")
 @cli.argument('-a', '--as', dest='accept', arg_only=True, choices=['create'], help="request the as=create alternative form of the resource")
-@cli.argument('resource_type', arg_only=True, help='type of resource', metavar="RESOURCE_TYPE", choices=[choice for group in [[singular(type), RESOURCES[type].abbreviation] for type in RESOURCES.keys()] for choice in group])
+@cli.argument('-e', '--embed', arg_only=True, nargs='+',
+              help="applies to 'get network': optionally embed space-sep list of resource types or 'all' of a network's resource collections",
+              choices=[plural(type) for type in EMBED_NET_RESOURCES.keys()].extend('all'))
+@cli.argument('resource_type', arg_only=True, help='type of resource', metavar="RESOURCE_TYPE",
+              choices=[choice for group in [[singular(type), RESOURCES[type].abbreviation] for type in RESOURCES.keys()] for choice in group])
 @cli.subcommand('get a single resource by type and query')
-def get(cli, echo: bool = True, embed='all', spinner: object = None):
+def get(cli, echo: bool = True, spinner: object = None):
     """
     Get a single resource as YAML or JSON.
 
-    :param echo: False allows capture instead of print the match
-    :param embed: False avoids expensive operations when we just need a shallow view
+    :param echo: False allows the caller to capture the return instead of printing the match
     """
     if RESOURCE_ABBREV.get(cli.args.resource_type):
         cli.args.resource_type = singular(RESOURCE_ABBREV[cli.args.resource_type].name)
     if not cli.config.general.verbose and cli.args.output in ["yaml", "json"]:    # don't change level if output=text
         cli.log.setLevel(logging.WARN)                                            # don't emit INFO messages to stdout because they will break deserialization
     if cli.args.accept and not MUTABLE_NET_RESOURCES.get(plural(cli.args.resource_type)):
-        logging.warning("ignoring --as=create becuase it is applicable only to mutable resources in the network domain")
+        logging.warning("ignoring --as=create because it is applicable only to mutable resources in the network domain")
         cli.args['accept'] = None
     match = {}
     matches = []
@@ -458,39 +490,35 @@ def get(cli, echo: bool = True, embed='all', spinner: object = None):
                 if len(query_keys) > 1:
                     query_keys.remove('id')
                     cli.log.warn(f"using 'id' only, ignoring params: '{', '.join(query_keys)}'")
-                match = organization.get_network(network_id=cli.args.query['id'], embed=embed, accept=cli.args.accept)
+                match = organization.get_network(network_id=cli.args.query['id'], embed=cli.args.embed, accept=cli.args.accept)
             else:
                 if cli.config.general.network_group and not cli.config.general.network:
                     network_group = use_network_group(
                         cli,
                         organization,
-                        group=cli.config.general.network_group,
-                        )
+                        group=cli.config.general.network_group)
                     matches = organization.find_networks_by_group(network_group.id, **cli.args.query)
                 elif cli.config.general.network:
                     network, network_group = use_network(
                         cli,
                         organization=organization,
-                        network_name=cli.config.general.network,
-                        )
-                    match = organization.get_network(network_id=network.id, embed=embed, accept=cli.args.accept)
+                        network_name=cli.config.general.network)
+                    match = organization.get_network(network_id=network.id, embed=cli.args.embed, accept=cli.args.accept)
                 else:
                     matches = organization.find_networks_by_organization(**cli.args.query)
                 if len(matches) == 1:
                     network, network_group = use_network(
                         cli,
                         organization=organization,
-                        network_name=matches[0]['name'],
-                        )
-                    match = organization.get_network(network_id=network.id, embed=embed, accept=cli.args.accept)
+                        network_name=matches[0]['name'])
+                    match = organization.get_network(network_id=network.id, embed=cli.args.embed, accept=cli.args.accept)
         else:                                                 # is a resource in the network domain
             if cli.config.general.network:
                 network, network_group = use_network(
                     cli,
                     organization=organization,
                     group=cli.config.general.network_group,   # None unless configured
-                    network_name=cli.config.general.network,
-                    )
+                    network_name=cli.config.general.network)
             else:
                 cli.log.error("need --network=ACMENet")
                 sysexit(1)
@@ -537,15 +565,27 @@ def get(cli, echo: bool = True, embed='all', spinner: object = None):
             if cli.args.output in ["yaml", "text"]:
                 if cli.config.general.color:
                     highlighted = highlight(yaml_dumps(filtered_match, indent=4), yaml_lexer, Terminal256Formatter(style=cli.config.general.style))
-                    cli.echo(highlighted)
+                    try:
+                        cli.echo(highlighted)
+                    except ValueError:
+                        print(highlighted)
                 else:
-                    cli.echo(yaml_dumps(filtered_match, indent=4))
+                    try:
+                        cli.echo(yaml_dumps(filtered_match, indent=4))
+                    except ValueError:
+                        print(yaml_dumps(filtered_match, indent=4))
             elif cli.args.output == "json":
                 if cli.config.general.color:
                     highlighted = highlight(json_dumps(filtered_match, indent=4), json_lexer, Terminal256Formatter(style=cli.config.general.style))
-                    cli.echo(highlighted)
+                    try:
+                        cli.echo(highlighted)
+                    except ValueError:
+                        print(highlighted)
                 else:
-                    cli.echo(json_dumps(filtered_match, indent=4))
+                    try:
+                        cli.echo(json_dumps(filtered_match, indent=4))
+                    except ValueError:
+                        print(json_dumps(filtered_match, indent=4))
     elif len(matches) == 0:
         cli.log.warn(f"found no {cli.args.resource_type} by '{', '.join(query_keys)}'")
         sysexit(1)
@@ -561,7 +601,8 @@ def get(cli, echo: bool = True, embed='all', spinner: object = None):
 @cli.argument('-k', '--keys', arg_only=True, action=StoreListKeys, help="list of keys as a,b,c to print only selected keys (columns)")
 @cli.argument('-m', '--my-roles', arg_only=True, action='store_true', help="filter roles by caller identity")
 @cli.argument('-a', '--as', dest='accept', arg_only=True, choices=['create'], help="request the as=create alternative form of the resources")
-@cli.argument('resource_type', arg_only=True, help='type of resource', metavar="RESOURCE_TYPE", choices=[choice for group in [[type, RESOURCES[type].abbreviation] for type in RESOURCES.keys()] for choice in group])
+@cli.argument('resource_type', arg_only=True, help='type of resource', metavar="RESOURCE_TYPE",
+              choices=[choice for group in [[type, RESOURCES[type].abbreviation] for type in RESOURCES.keys()] for choice in group])
 @cli.subcommand(description='find a collection of resources by type and query')
 def list(cli, spinner: object = None):
     """Find resources as lists."""
@@ -646,7 +687,7 @@ def list(cli, spinner: object = None):
                            'edgeRouterAttributes', 'serviceAttributes', 'endpointAttributes',
                            'status', 'zitiId', 'provider', 'locationCode', 'ipAddress', 'networkVersion',
                            'active', 'default', 'region', 'size', 'attributes', 'email', 'productVersion',
-                           'state']
+                           'state', 'address', 'binding']
         valid_keys = set(matches[0].keys()) & set(default_columns)
 
     if valid_keys:
@@ -683,19 +724,32 @@ def list(cli, spinner: object = None):
     elif cli.args.output == "yaml":
         if cli.config.general.color:
             highlighted = highlight(yaml_dumps(filtered_matches, indent=4), yaml_lexer, Terminal256Formatter(style=cli.config.general.style))
-            cli.echo(highlighted)
+            try:
+                cli.echo(highlighted)
+            except ValueError:
+                print(highlighted)
         else:
-            cli.echo(yaml_dumps(filtered_matches, indent=4))
+            try:
+                cli.echo(yaml_dumps(filtered_matches, indent=4))
+            except ValueError:
+                print(yaml_dumps(filtered_matches, indent=4))
     elif cli.args.output == "json":
         if cli.config.general.color:
             highlighted = highlight(json_dumps(filtered_matches, indent=4), json_lexer, Terminal256Formatter(style=cli.config.general.style))
-            cli.echo(highlighted)
+            try:
+                cli.echo(highlighted)
+            except ValueError:
+                print(highlighted)
         else:
-            cli.echo(json_dumps(filtered_matches, indent=4))
+            try:
+                cli.echo(json_dumps(filtered_matches, indent=4))
+            except ValueError:
+                print(json_dumps(filtered_matches, indent=4))
 
 
 @cli.argument('query', arg_only=True, action=StoreDictKeyPair, nargs='?', help="query params as k=v,k=v comma-separated pairs")
-@cli.argument('resource_type', arg_only=True, help='type of resource', choices=[choice for group in [[singular(type), RESOURCES[type].abbreviation] for type in RESOURCES.keys()] for choice in group])
+@cli.argument('resource_type', arg_only=True, help='type of resource',
+              choices=[choice for group in [[singular(type), RESOURCES[type].abbreviation] for type in RESOURCES.keys()] for choice in group])
 @cli.subcommand('delete a single resource by type and query')
 def delete(cli):
     """Delete a resource in the network domain."""
@@ -704,10 +758,11 @@ def delete(cli):
     if MUTABLE_RESOURCE_ABBREV.get(cli.args.resource_type):
         cli.args['resource_type'] = singular(MUTABLE_RESOURCE_ABBREV[cli.args.resource_type].name)
     cli.args['accept'] = None
+    cli.args['embed'] = None
     cli.config.general['wait'] = 0
     spinner.text = f"Finding {cli.args.resource_type} {'by' if query_keys else '...'} {', '.join(query_keys)}"
     with spinner:
-        match, network, network_group, organization = get(cli, echo=False, embed=None, spinner=spinner)
+        match, network, network_group, organization = get(cli, echo=False, spinner=spinner)
     if cli.args.resource_type == 'network':
         try:
             delete_confirmed = False
@@ -1119,7 +1174,7 @@ def use_network_group(cli, organization: object, group: str = None, spinner: obj
     with spinner:
         network_group = NetworkGroup(
             organization,
-            group=group if group else None,
+            group=group,
         )
     spinner.succeed(f"Configured network group '{network_group.name}'")
     cli.log.debug(f"network group is {network_group.name}")
