@@ -330,10 +330,8 @@ def find_generic_resources(url: str, headers: dict, embedded: str = None, proxie
     # normalize output with a default sort param
     if not params.get('sort'):
         params["sort"] = "name,asc"
-    # workaround sort param bug in MOP-18018
-    if resource_type.name in ['identities', 'user-identities', 'api-account-identities']:
-        del params['sort']
-    elif resource_type.name == 'hosts':      # workaround sort param bug in MOP-17863
+    # workaround sort param bugs in MOP-18018, MOP-17863, MOP-18178
+    if resource_type.name in ['identities', 'user-identities', 'api-account-identities', 'hosts', 'terminators']:
         del params['sort']
 
     # only get one page of the requested size, else default page size and all pages
@@ -395,8 +393,13 @@ def find_generic_resources(url: str, headers: dict, embedded: str = None, proxie
             yield yield_page
 
             # then yield subsequent pages, if applicable
-            if get_all_pages:         # this is False if param 'page' or 'size' to stop recursion or get a single page
-                for next_page in range(1, total_pages):  # first page is 0
+            if get_all_pages:           # this is False if param 'page' or 'size' to stop recursion or get a single page
+                if resource_type.name == 'network-groups':
+                    params['page'] = 1  # workaround API bug https://netfoundry.atlassian.net/browse/MOP-17890
+                    next_range_lower, next_range_upper = params['page'] + 2, total_pages + 1
+                else:
+                    next_range_lower, next_range_upper = params['page'] + 1, total_pages
+                for next_page in range(next_range_lower, next_range_upper):  # first page is 0 unless network-groups which are 1-based
                     params['page'] = next_page
                     try:
                         # recurse
@@ -706,7 +709,28 @@ RESOURCES = {
         mutable=True,
         embeddable=True,
         create_responses=["ACCEPTED"],
-    )
+    ),
+    'config-types': ResourceType(
+        name='config-types',
+        domain='network',
+        mutable=True,
+        embeddable=True,
+        create_responses=["ACCEPTED"],
+    ),
+    'configs': ResourceType(
+        name='configs',
+        domain='network',
+        mutable=True,
+        embeddable=True,
+        create_responses=["ACCEPTED"],
+    ),
+    'terminators': ResourceType(
+        name='terminators',
+        domain='network',
+        mutable=True,
+        embeddable=True,
+        create_responses=["ACCEPTED"],
+    ),
 }
 
 # TODO: [MOP-13441] associate locations with a short list of major geographic regions / continents
